@@ -1,6 +1,6 @@
 package HTML::Template;
 
-$HTML::Template::VERSION = '1.6';
+$HTML::Template::VERSION = '1.8';
 
 =head1 NAME
 
@@ -121,7 +121,7 @@ In order to realize a dramatic savings in bandwidth, the standard
 (non-comment) tags will be used throughout the rest of this
 documentation.
 
-=head2 <TMPL_VAR ?ESCAPE=1? NAME="PARAMETER_NAME">
+=head2 <TMPL_VAR NAME="PARAMETER_NAME">
 
 The <TMPL_VAR> tag is very simple.  For each <TMPL_VAR> tag in the
 template you call $template->param(PARAMETER_NAME => "VALUE").  When
@@ -260,7 +260,8 @@ Inside a <TMPL_LOOP>, the only variables that are usable are the ones
 from the <TMPL_LOOP>.  The variables in the outer blocks are not
 visible within a template loop.  For the computer-science geeks among
 you, a <TMPL_LOOP> introduces a new scope much like a perl subroutine
-call.  Unlike perl, there are no global variables in the templates.
+call.  If you want your variables to be global you can use
+'global_vars' option to new described below.
 
 =head2 <TMPL_INCLUDE NAME="filename.tmpl">
 
@@ -591,10 +592,10 @@ not __INNER__.
 path - you can set this variable with a list of paths to search for
 files specified with the "filename" option to new() and for files
 included with the <TMPL_INCLUDE> tag.  This list is only consulted
-when the filename is relative - i.e. does not begin with a '/'.  The
-HTML_TEMPLATE_ROOT environment variable is always tried first if it
-exists.  In the case of a <TMPL_INCLUDE> file, the path to the
-including file is also tried before path is consulted.
+when the filename is relative.  The HTML_TEMPLATE_ROOT environment
+variable is always tried first if it exists.  In the case of a
+<TMPL_INCLUDE> file, the path to the including file is also tried
+before path is consulted.
 
 Example:
 
@@ -604,6 +605,9 @@ Example:
                                                ]
                                       );
 
+NOTE: the paths in the path list must be expressed as UNIX paths,
+separated by the forward-slash character ('/').
+
 =item *
 
 max_includes - set this variable to determine the maximum depth that
@@ -613,6 +617,24 @@ to 0 to disable this protection.
 
 =item *
 
+global_vars - normally variables declared outside a loop are not
+available inside a loop.  This option makes <TMPL_VAR>s like global
+variables in Perl - they have unlimited scope.  This option also
+affects <TMPL_IF> and <TMPL_UNLESS>.
+
+Example:
+
+  This is a normal variable: <TMPL_VAR NORMAL>.<P>
+
+  <TMPL_LOOP NAME=FROOT_LOOP>
+     Here it is inside the loop: <TMPL_VAR NORMAL><P>
+  </TMPL_LOOP>
+
+Normally this wouldn't work as expected, since <TMPL_VAR NORMAL>'s
+value outside the loop is not available inside the loop.
+
+=item *
+
 vanguard_compatibility_mode - if set to 1 the module will expect to
 see <TMPL_VAR>s that look like %NAME% in addition to the standard
 syntax.  Also sets die_on_bad_params => 0.  If you're not at Vanguard
@@ -653,31 +675,30 @@ memory usage to STDERR.  Requires the GTop module.  Defaults to 0.
 use integer; # no floating point math so far!
 use strict; # and no funny business, either.
 
+use Carp; # generate better errors with more context
+use File::Spec; # generate paths that work on all platforms
+
 # define accessor constants used to improve readability of array
-# accesses into "objects".
+# accesses into "objects".  I used to use 'use constant' but that
+# seems to cause occasional irritating warnings in older Perls.
 package HTML::Template::LOOP;
 sub TEMPLATE_HASH { 0; }
 sub PARAM_SET { 1 };
 
 package HTML::Template::COND;
-use constant VARIABLE => 0;
-use constant VARIABLE_TYPE => 1;
-use constant VARIABLE_TYPE_VAR => 0;
-use constant VARIABLE_TYPE_LOOP => 1;
-use constant JUMP_IF_TRUE => 2;
-use constant JUMP_ADDRESS => 3;
-use constant WHICH => 4;
-use constant WHICH_IF => 0;
-use constant WHICH_UNLESS => 1;
+sub VARIABLE { 0 };
+sub VARIABLE_TYPE { 1 };
+sub VARIABLE_TYPE_VAR { 0 };
+sub VARIABLE_TYPE_LOOP { 1 };
+sub JUMP_IF_TRUE { 2 };
+sub JUMP_ADDRESS { 3 };
+sub WHICH { 4 };
+sub WHICH_IF { 0 };
+sub WHICH_UNLESS { 1 };
 
 # back to the main package scope.
 package HTML::Template;
 
-use vars qw($NOOP $ESCAPE $URLESCAPE);
-$NOOP = HTML::Template::NOOP->new();
-$ESCAPE = HTML::Template::ESCAPE->new();
-$URLESCAPE = HTML::Template::URLESCAPE->new();
-
 # open a new template and return an object handle
 sub new {
   my $pkg = shift;
@@ -710,11 +731,12 @@ sub new {
                ipc_mode => 0666,
                ipc_segment_size => 65536,
                ipc_max_size => 0,
+               global_vars => 0,
               );
   
   # load in options supplied to new()
   for (my $x = 0; $x <= $#_; $x += 2) {
-    defined($_[($x + 1)]) or die "HTML::Template->new() called with odd number of option parameters - should be of the form option => value";
+    defined($_[($x + 1)]) or croak("HTML::Template->new() called with odd number of option parameters - should be of the form option => value");
     $options->{lc($_[$x])} = $_[($x + 1)]; 
   }
 
@@ -734,10 +756,10 @@ sub new {
 
   # handle the "type", "source" parameter format (does anyone use it?)
   if (exists($options->{type})) {
-    exists($options->{source}) or die "HTML::Template->new() called with 'type' parameter set, but no 'source'!";
+    exists($options->{source}) or croak("HTML::Template->new() called with 'type' parameter set, but no 'source'!");
     ($options->{type} eq 'filename' or $options->{type} eq 'scalarref' or
      $options->{type} eq 'arrayref') or
-       die "HTML::Template->new() : type parameter must be set to 'filename', 'arrayref' or 'scalarref'!";
+       croak("HTML::Template->new() : type parameter must be set to 'filename', 'arrayref' or 'scalarref'!");
     $options->{$options->{type}} = $options->{source};
     delete $options->{type};
     delete $options->{source};
@@ -757,7 +779,7 @@ sub new {
   # make sure objects in associate area support param()
   foreach my $object (@{$options->{associate}}) {
     defined($object->can('param')) or
-      die "HTML::Template->new called with associate option, containing object of type " . ref($object) . " which lacks a param() method!";
+      croak("HTML::Template->new called with associate option, containing object of type " . ref($object) . " which lacks a param() method!");
   } 
 
 
@@ -767,14 +789,14 @@ sub new {
   exists($options->{arrayref}) and $source_count++;
   exists($options->{scalarref}) and $source_count++;
   if ($source_count != 1) {
-    die "HTML::Template->new called with multiple (or no) template sources specified!  A valid call to new() has exactly one filename => 'file' OR exactly one scalarRef => \\\$scalar OR exactly one arrayRef => \\\@array";    
+    croak("HTML::Template->new called with multiple (or no) template sources specified!  A valid call to new() has exactly one filename => 'file' OR exactly one scalarRef => \\\$scalar OR exactly one arrayRef => \\\@array");    
   }
 
   # do some memory debugging - this is best started as early as possible
   if ($options->{memory_debug}) {
     # memory_debug needs GTop
     eval { require 'GTop.pm'; };
-    die "Could not load GTop.  You must have GTop installed to use HTML::Template in memory_debug mode.  The error was: $@"
+    croak("Could not load GTop.  You must have GTop installed to use HTML::Template in memory_debug mode.  The error was: $@")
       if ($@);
     $self->{gtop} = GTop->new();
     $self->{proc_mem} = $self->{gtop}->proc_mem($$);
@@ -784,7 +806,7 @@ sub new {
   if ($options->{shared_cache}) {
     # shared_cache needs some extra modules loaded
     eval { require 'IPC/SharedCache.pm'; };
-    die "Could not load IPC::SharedCache.  You must have IPC::SharedCache installed to use HTML::Template in shared_cache mode.  The error was: $@"
+    croak("Could not load IPC::SharedCache.  You must have IPC::SharedCache installed to use HTML::Template in shared_cache mode.  The error was: $@")
       if ($@);
 
     # initialize the shared cache
@@ -840,7 +862,7 @@ sub _new_from_loop {
   
   # load in options supplied to new()
   for (my $x = 0; $x <= $#_; $x += 2) { 
-    defined($_[($x + 1)]) or die "HTML::Template->new() called with odd number of option parameters - should be of the form option => value";
+    defined($_[($x + 1)]) or croak("HTML::Template->new() called with odd number of option parameters - should be of the form option => value");
     $options->{lc($_[$x])} = $_[($x + 1)]; 
   }
 
@@ -970,7 +992,7 @@ sub _commit_to_cache {
   my $filepath = $options->{filepath};
   if (not defined $filepath) {
     $filepath = $self->_find_file($options->{filename});
-    die "HTML::Template->new() : Cannot open included file $options->{filename} : file not found."
+    confess("HTML::Template->new() : Cannot open included file $options->{filename} : file not found.")
       unless defined($filepath);
     $options->{filepath} = $filepath;   
   }
@@ -1074,30 +1096,30 @@ sub _find_file {
   my $filepath;
 
   # first check for a full path
-  return $filename 
-      if (($filename =~ /^\//) and (-e $filename));
+  return File::Spec->canonpath($filename)
+    if (File::Spec->file_name_is_absolute($filename) and (-e $filename));
 
   # try the extra_path if one was specified
   if (defined($extra_path)) {
     $extra_path->[$#{$extra_path}] = $filename;
-    $filepath = join('/', @$extra_path);
-    return $filepath if -e $filepath;
+    $filepath = File::Spec->canonpath(File::Spec->catfile(@$extra_path));
+    return File::Spec->canonpath($filepath) if -e $filepath;
   }
 
   # try pre-prending HTML_Template_Root
   if (exists($ENV{HTML_TEMPLATE_ROOT})) {
-    $filepath = join('/', split(/\/+/, $ENV{HTML_TEMPLATE_ROOT}), $filename);
-    return $filepath if -e $filepath;
+    $filepath =  File::Spec->catfile($ENV{HTML_TEMPLATE_ROOT}, $filename);
+    return File::Spec->canonpath($filepath) if -e $filepath;
   }
 
   # try "path" option list..
   foreach my $path (@{$options->{path}}) {
-    $filepath = join('/', split(/\/+/, $path), $filename);
-    return $filepath if -e $filepath;
+    $filepath = File::Spec->canonpath(File::Spec->catfile($path, $filename));
+    return File::Spec->canonpath($filepath) if -e $filepath;
   }
 
   # try even a relative path from the current directory...
-  return $filename if -e $filename;
+  return File::Spec->canonpath($filename) if -e $filename;
   
   return undef;
 }
@@ -1150,12 +1172,12 @@ sub _init_template {
     my $filepath = $options->{filepath};
     if (not defined $filepath) {
       $filepath = $self->_find_file($options->{filename});
-      die "HTML::Template->new() : Cannot open included file $options->{filename} : file not found."
+      confess("HTML::Template->new() : Cannot open included file $options->{filename} : file not found.")
         unless defined($filepath);
       # we'll need this for future reference - to call stat() for example.
       $options->{filepath} = $filepath;   
     }
-    die "HTML::Template->new() : Cannot open included file $options->{filename} : $!"
+    confess("HTML::Template->new() : Cannot open included file $options->{filename} : $!")
       unless defined(open(TEMPLATE, $filepath));
     
     # read into the array, note the mtime for the record
@@ -1182,7 +1204,7 @@ sub _init_template {
 
     delete($options->{arrayref});
   } else {
-    die("HTML::Template : Need to call new with filename, scalarref or arrayref parameter specified.");
+    confess("HTML::Template : Need to call new with filename, scalarref or arrayref parameter specified.");
   }
 
   print STDERR "### HTML::Template Memory Debug ### END INIT_TEMPLATE ", $self->{proc_mem}->size(), "\n"
@@ -1205,8 +1227,8 @@ sub _parse {
   # setup the stacks and maps - they're accessed by typeglobs that
   # reference the top of the stack.  They are masked so that a loop
   # can transparently have its own versions.
-  use vars qw(@pstack %pmap @ifstack @ucstack);
-  local (*pstack, *ifstack, *pmap, *ucstack);
+  use vars qw(@pstack %pmap @ifstack @ucstack %top_pmap);
+  local (*pstack, *ifstack, *pmap, *ucstack, *top_pmap);
   
   # the pstack is the array of scalar refs (plain text from the
   # template file), VARs, LOOPs, IFs and ELSEs that output() works on
@@ -1221,6 +1243,7 @@ sub _parse {
   # pmap at all!
   my @pmaps = ({});
   *pmap = $pmaps[0];
+  *top_pmap = $pmaps[0];
   $self->{param_map} = $pmaps[0];
 
   # the ifstack is a temporary stack containing pending ifs and elses
@@ -1254,6 +1277,10 @@ sub _parse {
                 ]);
   (*fname, *fcounter, *fmax) = \ ( @{$fstack[0]} );
 
+  my $NOOP = HTML::Template::NOOP->new();
+  my $ESCAPE = HTML::Template::ESCAPE->new();
+  my $URLESCAPE = HTML::Template::URLESCAPE->new();
+
   # all the tags that need NAMEs:
   my %need_names = map { $_ => 1 } 
     qw(TMPL_VAR TMPL_LOOP TMPL_IF TMPL_UNLESS TMPL_INCLUDE);
@@ -1410,6 +1437,8 @@ sub _parse {
           } else {
             $var = HTML::Template::VAR->new();
             $pmap{$name} = $var;
+            $top_pmap{$name} = HTML::Template::VAR->new()
+              if $options->{global_vars} and not exists $top_pmap{$name};
           }
 
           # push text coming before the tag onto the pstack,
@@ -1422,10 +1451,13 @@ sub _parse {
 
           # if ESCAPE was set, push an ESCAPE op on the stack before
           # the variable.  output will handle the actual work.
-          push(@pstack, $ESCAPE)
-            if ($escape and length($escape) == 4);
-          push(@pstack, $URLESCAPE)
-            if ($escape and length($escape) == 3);
+          if ($escape) {
+            if ($escape =~ /^"?[Uu][Rr][Ll]"?$/) {
+              push(@pstack, $URLESCAPE);
+            } else {
+              push(@pstack, $ESCAPE);
+            }
+          }
           
           push(@pstack, $var);
 
@@ -1512,6 +1544,8 @@ sub _parse {
               $uc->[HTML::Template::COND::VARIABLE] = $pmap{$var};
             } else {
               $pmap{$var} = HTML::Template::VAR->new();
+              $top_pmap{$var} = HTML::Template::VAR->new()
+                if $options->{global_vars} and not exists $top_pmap{$var};
               $uc->[HTML::Template::COND::VARIABLE] = $pmap{$var};
             }
             if (ref($pmap{$var}) eq 'HTML::Template::VAR') {
@@ -1533,12 +1567,12 @@ sub _parse {
           *ifstack = $ifstacks[$#ifstacks];
           pop(@ucstacks);
           *ucstack = $ucstacks[$#ucstacks];
-          
+
           # instantiate the sub-Template, feeding it parse_stack and
           # param_map.  This means that only the enclosing template
           # does _parse() - sub-templates get their parse_stack and
           # param_map fed to them already filled in.
-          $loop->[HTML::Template::LOOP::TEMPLATE_HASH]{$starts_at} 
+          $loop->[HTML::Template::LOOP::TEMPLATE_HASH]{$starts_at}             
             = HTML::Template->_new_from_loop(
                                              parse_stack => $parse_stack,
                                              param_map => $param_map,
@@ -1756,6 +1790,8 @@ sub _parse {
       $uc->[HTML::Template::COND::VARIABLE] = $pmap{$var};
     } else {
       $pmap{$var} = HTML::Template::VAR->new();
+      $top_pmap{$var} = HTML::Template::VAR->new()
+        if $options->{global_vars} and not exists $top_pmap{$var};
       $uc->[HTML::Template::COND::VARIABLE] = $pmap{$var};
     }
     if (ref($pmap{$var}) eq 'HTML::Template::VAR') {
@@ -1764,7 +1800,9 @@ sub _parse {
       $uc->[HTML::Template::COND::VARIABLE_TYPE] = HTML::Template::COND::VARIABLE_TYPE_LOOP;
     }
   }
-              
+
+  # globalize vars?
+  $self->_globalize_vars() if ($options->{global_vars});
 
   # want a stack dump?
   if ($options->{stack_debug}) {
@@ -1776,6 +1814,22 @@ sub _parse {
   delete $self->{template};
 }
 
+# a recursive sub that associates each loop with the loops above
+# (treating the top-level as a loop)
+sub _globalize_vars {
+  my $self = shift;
+  
+  # associate with the loop (and top-level templates) above in the tree.
+  push(@{$self->{options}{associate}}, @_);
+  
+  # recurse down into the template tree, adding ourself to the end of
+  # list.
+  push(@_, $self);
+  map { $_->_globalize_vars(@_) } 
+    map {values %{$_->[HTML::Template::LOOP::TEMPLATE_HASH]}}
+      grep { ref($_) eq 'HTML::Template::LOOP'} @{$self->{parse_stack}};
+}
+
 =head2 param
 
 param() can be called in a number of ways
@@ -1873,7 +1927,7 @@ sub param {
     
     # check for parameter existence 
     $options->{die_on_bad_params} and !exists($param_map->{$param}) and
-      die("HTML::Template : Attempt to get nonexistent parameter $param : (die_on_bad_params set => 1)");
+      croak("HTML::Template : Attempt to get nonexistent parameter '$param' - this parameter name doesn't match any declarations in the template file : (die_on_bad_params set => 1)");
     
     return undef unless (exists($param_map->{$param}) and
                          defined($param_map->{$param}));
@@ -1884,14 +1938,14 @@ sub param {
   } 
 
   if (!scalar(@_)) {
-    die "HTML::Template->param() : Single reference arg to param() must be a hash-ref!  You gave me a $type." 
+    croak("HTML::Template->param() : Single reference arg to param() must be a hash-ref!  You gave me a $type.")
       unless $type eq 'HASH' or (ref($first) and $first->isa('HASH'));  
     push(@_, %$first);
   } else {
     unshift(@_, $first);
   }
   
-  die "HTML::Template->param() : You gave me an odd number of parameters to param()!"
+  croak("HTML::Template->param() : You gave me an odd number of parameters to param()!")
     unless ((@_ % 2) == 0);
 
   # strangely, changing this to a "while(@_) { shift, shift }" type
@@ -1903,7 +1957,7 @@ sub param {
     
     # check that this param exists in the template
     $options->{die_on_bad_params} and !exists($param_map->{$param}) and
-      die("HTML::Template : Attempt to set nonexistent parameter $param : (die_on_bad_params => 1)");
+      croak("HTML::Template : Attempt to set nonexistent parameter '$param' - this parameter name doesn't match any declarations in the template file : (die_on_bad_params => 1)");
     
     # if we're not going to die from bad param names, we need to ignore
     # them...
@@ -1914,16 +1968,18 @@ sub param {
     my $value_type = ref($value);
     if (defined($value_type) and length($value_type) and ($value_type eq 'ARRAY' or ((ref($value) !~ /^(CODE)|(HASH)|(SCALAR)$/) and $value->isa('ARRAY')))) {
       (ref($param_map->{$param}) eq 'HTML::Template::LOOP') or
-        die "HTML::Template::param() : attempt to set parameter $param with an array ref - parameter is not a TMPL_LOOP!";
+        croak("HTML::Template::param() : attempt to set parameter '$param' with an array ref - parameter is not a TMPL_LOOP!");
       $param_map->{$param}[HTML::Template::LOOP::PARAM_SET] = [@{$value}];
     } else {
       (ref($param_map->{$param}) eq 'HTML::Template::VAR') or
-        die "HTML::Template::param() : attempt to set parameter $param with a scalar - parameter is not a TMPL_VAR!";
+        croak("HTML::Template::param() : attempt to set parameter '$param' with a scalar - parameter is not a TMPL_VAR!");
       ${$param_map->{$param}} = $value;
     }
   }
 }
 
+=pod
+
 =head2 clear_params()
 
 Sets all the parameters to undef.  Useful internally, if nowhere else!
@@ -1948,7 +2004,7 @@ sub associateCGI {
   my $self = shift;
   my $cgi  = shift;
   (ref($cgi) eq 'CGI') or
-    die("Warning! non-CGI object was passed to HTML::Template::associateCGI()!\n");
+    croak("Warning! non-CGI object was passed to HTML::Template::associateCGI()!\n");
   push(@{$self->{options}{associate}}, $cgi);
   return 1;
 }
@@ -2028,8 +2084,11 @@ sub output {
     } elsif ($type eq 'HTML::Template::VAR') {
       defined($$line) and $result .= $$line;
     } elsif ($type eq 'HTML::Template::LOOP') {
-      defined($line->[HTML::Template::LOOP::PARAM_SET]) and 
-        $result .= $line->output($x, $options->{loop_context_vars});
+      if (defined($line->[HTML::Template::LOOP::PARAM_SET])) {
+        eval { $result .= $line->output($x, $options->{loop_context_vars}); };
+        croak("HTML::Template->output() : fatal error in loop output : $@") 
+          if $@;
+      }
     } elsif ($type eq 'HTML::Template::COND') {
       if ($line->[HTML::Template::COND::JUMP_IF_TRUE]) {
         if ($line->[HTML::Template::COND::VARIABLE_TYPE] == HTML::Template::COND::VARIABLE_TYPE_VAR) {
@@ -2079,11 +2138,11 @@ sub output {
           for (0..255) { $URLESCAPE_MAP{chr($_)} = sprintf('%%%02X', $_); }
         }
         # do the translation (RFC 2396 ^uric)
-	$toencode =~ s/([^;\/?:@&=+\$,A-Za-z0-9\-_.!~*'()])/$URLESCAPE_MAP{$1}/g;
+        $toencode =~ s!([^a-zA-Z0-9_.\-])!$URLESCAPE_MAP{$1}!g;
         $result .= $toencode;
       }
     } else {
-      die "HTML::Template::output() : Unknown item in parse_stack : " . $type;
+      confess("HTML::Template::output() : Unknown item in parse_stack : " . $type);
     }
   }
 
@@ -2093,6 +2152,157 @@ sub output {
   return $result;
 }
 
+=pod
+
+=head2 query()
+
+This method allow you to get information about the template structure.
+It can be called in a number of ways.  The simplest usage of query is
+simply to check whether a parameter name exists in the template, using
+the C<name> option:
+
+  if ($template->query(name => 'foo')) {
+    # do something if a varaible of any type 
+    # named FOO is in the template
+  }
+
+This same usage returns the type of the parameter.  The type is the
+same as the tag minus the leading 'TMPL_'.  So, for example, a
+TMPL_VAR parameter returns 'VAR' from query().
+
+  if ($template->query(name => 'foo') eq 'VAR') {
+    # do something if FOO exists and is a TMPL_VAR
+  }
+
+Note that the variables associated with TMPL_IFs and TMPL_UNLESSs will
+be identified as 'VAR' unless they are also used in a TMPL_LOOP, in
+which case they will return 'LOOP'.
+
+C<query()> also allows you to get a list of parameters inside a loop
+(and inside loops inside loops).  Example loop:
+
+   <TMPL_LOOP NAME="EXAMPLE_LOOP">
+     <TMPL_VAR NAME="BEE">
+     <TMPL_VAR NAME="BOP">
+     <TMPL_LOOP NAME="EXAMPLE_INNER_LOOP">
+       <TMPL_VAR NAME="INNER_BEE">
+       <TMPL_VAR NAME="INNER_BOP">
+     </TMPL_LOOP>
+   </TMPL_LOOP>
+
+And some query calls:
+  
+  # returns 'LOOP'
+  $type = $template->query(name => 'EXAMPLE_LOOP');
+    
+  # returns ('bop', 'bee', 'example_inner_loop')
+  @param_names = $template->query(loop => 'EXAMPLE_LOOP');
+
+  # both return 'VAR'
+  $type = $template->query(name => ['EXAMPLE_LOOP', 'BEE']);
+  $type = $template->query(name => ['EXAMPLE_LOOP', 'BOP']);
+
+  # and this one returns 'LOOP'
+  $type = $template->query(name => ['EXAMPLE_LOOP', 
+                                    'EXAMPLE_INNER_LOOP']);
+  
+  # and finally, this returns ('inner_bee', 'inner_bop')
+  @inner_param_names = $template->query(loop => ['EXAMPLE_LOOP',
+                                                 'EXAMPLE_INNER_LOOP']);
+
+  # for non existent parameter names you get undef
+  # this returns undef.
+  $type = $template->query(name => 'DWEAZLE_ZAPPA');
+
+  # calling loop on a non-loop parameter name will cause an error.
+  # this dies:
+  $type = $template->query(loop => 'DWEAZLE_ZAPPA');
+
+As you can see above the C<loop> option returns a list of parameter
+names and both C<name> and C<loop> take array refs in order to refer
+to parameters inside loops.  It is an error to use C<loop> with a
+parameter that is not a loop.
+
+Note that all the names are returned in lowercase and the types are
+uppercase.
+
+Just like C<param()>, C<query()> with no arguements returns all the
+parameter names in the template at the top level.
+
+=cut
+
+sub query {
+  my $self = shift;
+  $self->{options}{debug} and print STDERR "### HTML::Template Debug ### query(", join(', ', @_), ")\n";
+
+  # the no-parameter case - return $self->param()
+  return $self->param() unless scalar(@_);
+  
+  croak("HTML::Template::query() : Odd number of parameters passed to query!")
+    if (scalar(@_) % 2);
+  croak("HTML::Template::query() : Wrong number of parameters passed to query - should be 2.")
+    if (scalar(@_) != 2);
+
+  my ($opt, $path) = (lc shift, shift);
+  croak("HTML::Template::query() : invalid parameter ($opt)")
+    unless ($opt eq 'name' or $opt eq 'loop');
+
+  # make path an array unless it already is
+  $path = [$path] unless (ref $path);
+
+  # find the param in question.
+  my @objs = $self->_find_param(@$path);
+  return undef unless scalar(@objs);
+  my ($obj, $type);
+
+  # do what the user asked with the object
+  if ($opt eq 'name') {
+    # we only look at the first one.  new() should make sure they're
+    # all the same.
+    ($obj, $type) = (shift(@objs), shift(@objs));
+    return undef unless defined $obj;
+    return 'VAR' if $type eq 'HTML::Template::VAR';
+    return 'LOOP' if $type eq 'HTML::Template::LOOP';
+    croak("HTML::Template::query() : unknown object ($type) in param_map!");
+
+  } elsif ($opt eq 'loop') {
+    my %results;
+    while(@objs) {
+      ($obj, $type) = (shift(@objs), shift(@objs));
+      croak("HTML::Template::query() : Search path [", join(', ', @$path), "] doesn't end in a TMPL_LOOP - it is an error to use the 'loop' option on a non-loop parameter.  To avoid this problem you can use the 'name' option to query() to check the type first.") 
+        unless ((defined $obj) and ($type eq 'HTML::Template::LOOP'));
+      
+      # SHAZAM!  This bit extracts all the parameter names from all the
+      # loop objects for this name.
+      map {$results{$_} = 1} map { keys(%{$_->{'param_map'}}) }
+        values(%{$obj->[HTML::Template::LOOP::TEMPLATE_HASH]});
+    }
+    # this is our loop list, return it.
+    return keys(%results);   
+  }
+}
+
+# a function that returns the object(s) corresponding to a given path and
+# its (their) ref()(s).  Used by query() in the obvious way.
+sub _find_param {
+  my $self = shift;
+  my $spot = lc shift;
+
+  # get the obj and type for this spot
+  my $obj = $self->{'param_map'}{$spot};
+  return unless defined $obj;
+  my $type = ref $obj;
+
+  # return if we're here or if we're not but this isn't a loop
+  return ($obj, $type) unless @_;
+  return unless ($type eq 'HTML::Template::LOOP');
+
+  # recurse.  this is a depth first seach on the template tree, for
+  # the algorithm geeks in the audience.
+  return map { $_->_find_param(@_) }
+    values(%{$obj->[HTML::Template::LOOP::TEMPLATE_HASH]});
+}
+
 # HTML::Template::VAR, LOOP, etc are *light* objects - their internal
 # spec is used above.  No encapsulation or information hiding is to be
 # assumed.
@@ -2124,20 +2334,29 @@ sub output {
   my $value_sets_array = $self->[PARAM_SET];
   next unless defined($value_sets_array);  
   
-  if ($loop_context_vars and defined($value_sets_array->[0])) {
-    $value_sets_array->[0]{__FIRST__} = 1; 
-    $value_sets_array->[$#{$value_sets_array}]{__LAST__} = 1;  
-    for (my $x = 1;$x < $#{$value_sets_array};$x++) {
-      $value_sets_array->[$x]{__INNER__} = 1;
-    }
-  }
-  
   my $result = '';
+  my $count = 0;
   foreach my $value_set (@$value_sets_array) {
-    $template->param($value_set);
+    if ($loop_context_vars) {
+      if ($count == 0) {
+	$value_set->{__FIRST__} = 1;
+      } elsif ($count == $#{$value_sets_array}) {
+	$value_set->{__LAST__} = 1;
+      } else {
+	$value_set->{__INNER__} = 1;
+      }
+    }
+    $template->param($value_set);    
     $result .= $template->output;
     $template->clear_params;
+    if ($loop_context_vars) {
+      delete $value_set->{__FIRST__};
+      delete $value_set->{__LAST__};
+      delete $value_set->{__INNER__};
+    }
+    $count++;
   }
+
   return $result;
 }
 
@@ -2214,9 +2433,11 @@ CVS SERVER section.
 
 This is the intended behavior.  <TMPL_LOOP> introduces a separate
 scope for <TMPL_VAR>s much like a subroutine call in Perl introduces a
-separate scope for "my" variables.  If you need to have a variable
-from the main template work inside a loop you'll need to manually
-provide the value for each iteration of the loop.
+separate scope for "my" variables.  
+
+If you want your <TMPL_VAR>s to be global you can set the
+'global_vars' option when you call new().  See above for documentation
+of the 'global_vars' new() option.
 
 5) Why do you use /[Tt]/ instead of /t/i?  It's so ugly!
 
@@ -2264,6 +2485,50 @@ code and look for "CACHE MISS" messages in the logs.
 
 Numbers, letters, '.', '/', '+', '-' and '_'.
 
+8) How can I execute a program from inside my template?  
+
+Short answer: you can't.  Longer answer: you shouldn't since this
+violates the fundamental concept behind HTML::Template - that design
+and code should be seperate.
+
+But, inevitably some people still want to do it.  At times it has even
+seemed that HTML::Template development might split over this issue, so
+I will attempt a compromise.  Here is a method you can use to allow
+your template authors to evaluate arbitrary perl scripts from within
+the template.
+
+First, tell all your designers that when they want to run a perl
+script named "program.pl" they should use a tag like:
+
+  <TMPL_VAR NAME="__execute_program.pl__">
+
+Then, have all your programmers call this subroutine instead of
+calling HTML::Template::new directly.  They still use the same
+parameters, but they also get the program execution.  
+
+  sub new_template {
+    # get the template object
+    my $template = HTML::Template->new(@_);
+    
+    # find program parameters and fill them in
+    my @params = $template->param();
+    for my $param (@params) {      
+       if ($param =~ /^__execute_(.*)__$/) {
+         $template->param($param, do($1));
+       }
+    }
+
+    # return the template object
+    return $template;
+  }
+
+The programs called in this way should return a string containing
+their output.  A more complicated subroutine could be written to
+capture STDOUT from the scripts, but this one is simple enough to
+include in the FAQ.  Another improvement would be to use query() to
+enable program execution inside loops.
+
+
 =head1 BUGS
 
 I am aware of no bugs - if you find one, join the mailing list and
@@ -2310,6 +2575,10 @@ provided by:
    Drew Taylor
    Tobias Brox
    Michael Lloyd
+   Simran Gambhir
+   Chris Houser <chouser@bluweb.com>
+   Larry Moore
+   Todd Larason
 
 Thanks!
 
@@ -2328,18 +2597,24 @@ htmltmpl-subscribe@lists.vm.com).
 =head1 LICENSE
 
 HTML::Template : A module for using HTML Templates with Perl
+Copyright (C) 2000 Sam Tregar (sam@tregar.com)
 
-Copyright (C) 1999 Sam Tregar (sam@tregar.com)
+This module is free software; you can redistribute it and/or modify it
+under the terms of either:
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or (at
-your option) any later version.
+a) the GNU General Public License as published by the Free Software
+Foundation; either version 1, or (at your option) any later version,
+or
 
-This program is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-General Public License for more details.
+b) the "Artistic License" which comes with this module.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See either
+the GNU General Public License or the Artistic License for more details.
+
+You should have received a copy of the Artistic License with this
+module, in the file ARTISTIC.  If not, I'll be glad to provide one.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
